/* main.c
 *
 * Copyright 2001-2003 Vesa Halttunen (Vesuri/dA JoRMaS)
 *
 * This file is part of JRm-core.
 *
 * JRm-core is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * JRm-core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JRm-core; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef WINDOWS
#include <Windows.h>
#define M_PI 3.14159265358979323846
#else
#include <sys/time.h>
#include <unistd.h>
#include <time.h>
#endif
#include <math.h>
#include <SDL.h>
#include <SDL_audio.h>
#include "production.h"
#include "buffer.h"
#include "effect.h"
#include "mp3.h"
#include "config.h"

#ifdef DEBUG
#define SKIP SONGPOSTOTIME(1, 0)
#else
#define SKIP 0
#endif

buffer screenbuffer;
long curtime=0;
int quit=0, timelinepos=0, sinus[1024];
effectlistentry *effectlist=NULL;
#ifdef WINDOWS
DWORD starttc;
#else
struct timeval tv, starttv;
struct timezone tz;
#endif

/* Used if save to disk is set */
int curframe=0;

int main(int argc, char **argv) {
#ifdef WORDS_BIGENDIAN
  SDL_AudioSpec desired={ 44100, AUDIO_S16MSB, 2, 0, 4096, 0, 0, playmp3, 0 };
#else
  SDL_AudioSpec desired={ 44100, AUDIO_S16LSB, 2, 0, 4096, 0, 0, playmp3, 0 };
#endif
  SDL_AudioSpec obtained;
  SDL_Surface *screen;
  SDL_Event event;
  effectlistentry *entry;
  int i, waitformouse=0;
  char *filename;
  struct playmp3data mp3data;
#ifdef WINDOWS
  unsigned int flags=SDL_SWSURFACE|SDL_FULLSCREEN;
#else
  unsigned int flags=SDL_SWSURFACE|SDL_FULLSCREEN|SDL_DOUBLEBUF;
#endif

  /* Check command line arguments */
  for(i=1; i<argc; i++) {
    /* Check if fullscreen mode was requested */
    if(strcmp(argv[i], "--windowed")==0)
      flags=SDL_SWSURFACE|SDL_DOUBLEBUF;
    else  if(strcmp(argv[i], "--waitformouse")==0)
      waitformouse=1;
    else
      fprintf(stderr, "Unsupported command line argument %s\n", argv[i]);
  }

  /* Get time of day for random seed */
#ifdef WINDOWS
  srand(GetTickCount());
#else
  srand(time(NULL));
#endif

  /* Initialize the production */
  production_startup();

  /* Initialize the SDL library */
  if(SDL_Init(SDL_INIT_VIDEO)<0) {
    fprintf(stderr, "Couldn't initialize SDL: %s\n", SDL_GetError());
    exit(1);
  }

  /* No audio if saving to disk */
  if(!savetodisk) {
    /* Open audio system */
    mp3data.time=production_mp3_time;
    mp3data.volume=production_mp3_volume;
    desired.userdata=&mp3data;
    if(SDL_OpenAudio(&desired, &obtained)<0) {
      fprintf(stderr, "Couldn't open audio: %s\n", SDL_GetError());
      exit(1);
    }
    
    /* Open MP3 files */
    for(i=0; i<production_mp3_files; i++) {
#ifdef DATADIR_PREFIX
      filename=(char *)calloc(strlen(DATADIR_PREFIX)+1+
			      strlen(production_name)+1+
			      strlen(production_mp3_filename[i])+1,
			      sizeof(char));
      sprintf(filename, "%s/%s/%s", DATADIR_PREFIX, production_name,
	      production_mp3_filename[i]);
      if(openmp3(filename, obtained.size, SKIP)) {
	free(filename);
#endif
	filename=(char *)calloc(strlen("data/")+
				strlen(production_mp3_filename[i])+1,
				sizeof(char));
	sprintf(filename, "data/%s", production_mp3_filename[i]);
	if(openmp3(filename, obtained.size, SKIP)) {
#ifdef DATADIR_PREFIX
	  fprintf(stderr, "Couldn't open MP3 file %s/%s/%s or data/%s\n",
		  DATADIR_PREFIX, production_name, production_mp3_filename[i],
		  production_mp3_filename[i]);
#else
	  fprintf(stderr, "Couldn't open MP3 file data/%s\n",
		  production_mp3_filename[i]);
#endif
	  exit(1);
	}
#ifdef DATADIR_PREFIX
      }
#endif
    }
  }

  /* Have a preference for 32-bit */
  if(!(screen=SDL_SetVideoMode(screenwidth, screenheight, 32, flags))) {
    fprintf(stderr, "Couldn't set %dx%dx32 video mode: %s\n",
	    screenwidth, screenheight, SDL_GetError());
    exit(1);
  }
  SDL_WM_SetCaption(production_window_title, production_window_title);
  SDL_ShowCursor(SDL_DISABLE);

  /* Wait for a mouse click if requested */
  if(waitformouse) {
    event.type=SDL_NOEVENT;
    while(event.type!=SDL_MOUSEBUTTONDOWN)
      SDL_WaitEvent(&event);
  }

  /* Create a buffer for the screen */
  screenbuffer.data=((unsigned int *)screen->pixels);
  screenbuffer.width=screenwidth;
  screenbuffer.height=screenheight;

  /* Create a sinus table */
  for(i=0; i<1024; i++)
    sinus[i]=(int)(sin(i*M_PI/512)*255);

  /* Get the initial time of day */
#ifdef WINDOWS
  starttc=GetTickCount();
  srand(starttc);
#else
  gettimeofday(&starttv, &tz);
  srand(starttv.tv_usec);
#endif

  /* No audio if saving to disk */
  if(!savetodisk)
    /* Start audio playback */
    SDL_PauseAudio(0);

  while(!(quit)) {
#ifdef SHOWFPS
    long oldcurtime;
#endif
    if(!savetodisk) {
      /* Get time of day for timing */
#ifdef SHOWFPS
      oldcurtime=curtime;
#endif
#ifdef WINDOWS
      curtime=GetTickCount()-starttc;
#else
      gettimeofday(&tv, &tz);
      curtime=((tv.tv_sec-starttv.tv_sec)*1000+(tv.tv_usec-starttv.tv_usec)/1000);
#endif
    } else
      curtime+=20;
    
#ifdef SHOWFPS
    printf("FPS: %f\n", 1000/(float)(curtime-oldcurtime));
#endif

    /* Check the timeline if we're supposed to do something */
    while(curtime+SKIP>=timeline[timelinepos].time) {
      if(timeline[timelinepos].job!=NULL) {
	timeline[timelinepos].job();
	timelinepos++;
      } else
	break;
    }

    /* Run all effects scheduled to run now */
    entry=effectlist;
    while(entry!=NULL) {
      if(entry->effect->parameters)
	entry->effect->parameters(entry->effect->data, curtime+SKIP);
      if(entry->effect->run)
	entry->effect->run(entry->effect->data, curtime+SKIP);
      entry=entry->next;
    }

    if(savetodisk) {
      filename=(char *)calloc(11, sizeof(char));
      snprintf(filename, 11, "%.5d.png", curframe+4114);
      fprintf(stderr, "Saving %s\n", filename);
      buffer_save(&screenbuffer, filename);
      curframe++;
      free(filename);
    }

    /* Double buffering */
    SDL_Flip(screen);

    /* Quit if window closed or escape pressed */
    while(SDL_PollEvent(&event)) {
      switch(event.type) {
      case SDL_QUIT:
	quit=1;
	break;
      case SDL_KEYUP:
	if(event.key.keysym.sym==SDLK_ESCAPE)
	  quit=1;
	break;
      default:
	break;
      }
    }
  }

  /* ..and die */
  if(!savetodisk)
    SDL_CloseAudio();
  SDL_Quit();
  if(!savetodisk)
    closemp3();
  production_shutdown();

  return 0;
}
