/* JRm-hata.h
 *
 * Copyright 2001-2002 dA JoRMaS
 *
 * This file is part of JRm-core.
 *
 * JRm-core is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * JRm-core is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JRm-core; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef WINDOWS
#define M_PI 3.14159265358979323846
#endif

/* Include everything that is needed by all parts */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "production.h"
#include "effect.h"
#include "copy.h"
#include "horizontalstretch.h"
#include "verticalstretch.h"
#include "add.h"
#include "horizontalstripes.h"
#include "verticalstripes.h"
#include "horizontalblur.h"
#include "verticalblur.h"

/* Song information */
#define BPM 120
#define PATTERNLENGTH 64

/* External definitions needed by all parts */
extern effectlistentry *effectlist;
extern buffer screenbuffer, **pictures;
extern int renderwidth, renderheight, screenwidth, screenheight;

/* Offsets to the picture table */
enum {
  ASSEMBLY,
  COLON,
  Y2K,
  HELSINKI,
  SHIPYARD1,
  DA,
  JORMAS,
  PRESENT,
  FACTORY1,
  FACTORY1OUTLINE,
  DESIGN1,
  DESIGN2,
  CROSSPLATFORM,
  DEMO,
  HATA,
  TRAFFIC1,
  TRAFFIC2,
  TRAFFIC3,
  TRAFFIC4,
  TRAFFIC5,
  TRAFFIC6,
  TRAFFIC7,
  TRAFFIC8,
  TRAFFIC,
  JAM,
  INSTANT,
  MOBILITY,
  TRAFFICBULLSHIT1,
  TRAFFICBULLSHIT2,
  BULLSHITWHITE,
  BULLSHITBLACK,
  TRAINBG,
  TRAIN,
  STRAIGHTELECTRIC,
  LOCOMOTIVES,
  TAKE,
  YOU,
  THERE,
  BUT,
  WHERE,
  FACTORY2,
  FACTORY2SMALL1,
  FACTORY2SMALL2,
  FACTORY2SMALL3,
  ENERGY,
  CRANE1,
  CRANE1SMALL1,
  CRANE1SMALL2,
  CRANE1SMALL3,
  MACHINERY,
  FACTORY3,
  FACTORY3SMALL1,
  FACTORY3SMALL2,
  FACTORY3SMALL3,
  ELECTRICITY,
  TRAFFIC9,
  TRAFFIC9SMALL1,
  TRAFFIC9SMALL2,
  TRAFFIC9SMALL3,
  TECHNOLOGY,
  LAPTOP1,
  LAPTOP2,
  LAPTOP3,
  LAPTOP4,
  LAPTOP5,
  LAPTOP6,
  LAPTOP7,
  LAPTOP8,
  LAPTOP9,
  LAPTOP10,
  LAPTOP11,
  LAPTOP12,
  LAPTOP13,
  LAPTOP14,
  LAPTOP15,
  LAPTOP16,
  MOTHERBOARD,
  MOBILE,
  COMPUTING,
  WHENEVER,
  WHEREEVER,
  PHONE,
  PHONEBOX,
  GLOBALCOVERAGE,
  FACTORY4,
  FACTORY4SMALL1,
  FACTORY4SMALL2,
  FACTORY4SMALL3,
  CRANE2,
  CRANE2SMALL1,
  CRANE2SMALL2,
  CRANE2SMALL3,
  POWERSTATION,
  POWERSTATIONSMALL1,
  POWERSTATIONSMALL2,
  POWERSTATIONSMALL3,
  TRAFFIC10,
  TRAFFIC10SMALL1,
  TRAFFIC10SMALL2,
  TRAFFIC10SMALL3,
  FUEL1,
  FUEL2,
  FUEL3,
  FUEL4,
  FUEL5,
  FUEL6,
  FUEL7,
  FUEL8,
  FUEL9,
  TECHNOLOGY2,
  POSSIBILITY,
  ANSWER,
  PROBLEMS,
  BINARY,
  EYE,
  WHAT,
  ABOUT,
  THE,
  PROBLEMS2,
  OFTHE,
  MIND,
  STAIRS1,
  STAIRS2,
  STAIRS3,
  STAIRS4,
  STAIRS5,
  STAIRS6,
  STAIRS7,
  STAIRS8,
  STAIRS9,
  STAIRS10,
  STAIRS11,
  STAIRS12,
  STAIRS13,
  STAIRS14,
  STAIRS15,
  FEAR,
  INSECURITY,
  LONELINESS,
  INSANITY,
  BASEMENT1,
  BASEMENT2,
  BASEMENT3,
  BASEMENT4,
  BASEMENT5,
  BASEMENT6,
  BASEMENT7,
  BASEMENT8,
  BASEMENT9,
  BASEMENT10,
  BASEMENT11,
  BASEMENT12,
  BASEMENT13,
  BASEMENT14,
  BASEMENT15,
  ELEVATOR1,
  ELEVATOR2,
  ELEVATOR3,
  ELEVATOR4,
  ELEVATOR5,
  ELEVATOR6,
  ELEVATOR7,
  ELEVATOR8,
  ELEVATOR9,
  VESURI1,
  VESURI2,
  VESURI3,
  VESURI4,
  VESURI5,
  VESURI6,
  VESURI7,
  VESURI8,
  LAPINKULTA,
  LAPINKULTASMALL1,
  LAPINKULTASMALL2,
  LAPINKULTASMALL3,
  LAPINKULTASMALL4,
  LAPINKULTASMALL5,
  LAPINKULTASMALL6,
  LAPINKULTASMALL7,
  LAPINKULTASMALL8,
  LAPINKULTASMALL9,
  LAPINKULTASMALL10,
  LAPINKULTASMALL11,
  LAPINKULTASMALL12,
  BEER,
  WATER1,
  WATER2,
  WATER3,
  WATER4,
  PEACEOFMIND,
  ESCAPINGREALITY,
  HELP1,
  HELP2,
  HELP3,
  HELP4,
  CREDITS,
  KODE,
  GFX,
  MUSIK,
  DESIGN,
  VESURI,
  THANKS,
  INSPIRATION,
  GMUNK,
  LIEVONEN,
  SUPPORT,
  VIRNES,
  JRM,
  DESIGN3,
  DESIGN4,
  DESIGN5,
  DESIGN6,
  DESIGN7,
  DESIGN8,
  SYMBOL,
  MIXER1,
  MIXER2,
  MIXER3,
  MIXER4,
  EPS,
  X51,
  X52,
  ENDSCREEN,
  TEMP
};

/* Part starting times */
#define PART_01_START SONGPOSTOTIME(1, 0)
#define PART_03_START SONGPOSTOTIME(3, 0)
#define PART_05_START SONGPOSTOTIME(5, 0)
#define PART_07_START SONGPOSTOTIME(6, 56)
#define PART_09_START SONGPOSTOTIME(9, 0)
#define PART_11_START SONGPOSTOTIME(11, 0)
#define PART_13_START SONGPOSTOTIME(13, 0)
#define PART_14_START SONGPOSTOTIME(14, 0)
#define PART_15_START SONGPOSTOTIME(15, 0)
#define PART_16_START SONGPOSTOTIME(16, 0)
#define PART_17_START SONGPOSTOTIME(17, 0)
#define PART_19_START SONGPOSTOTIME(19, 0)
#define PART_21_START SONGPOSTOTIME(21, 0)
#define PART_22_START SONGPOSTOTIME(22, 0)
#define PART_23_START SONGPOSTOTIME(23, 0)
#define PART_24_START SONGPOSTOTIME(24, 0)
#define PART_25_START SONGPOSTOTIME(25, 0)
#define PART_27_START SONGPOSTOTIME(27, 0)
#define PART_29_START SONGPOSTOTIME(29, 0)
#define PART_31_START SONGPOSTOTIME(31, 0)
#define PART_33_START SONGPOSTOTIME(33, 0)
#define PART_35_START SONGPOSTOTIME(35, 0)
#define PART_37_START SONGPOSTOTIME(37, 0)
#define PART_41_START SONGPOSTOTIME(41, 0)
#define PART_45_START SONGPOSTOTIME(45, 0)
#define PART_END SONGPOSTOTIME(47, 6)

/* Part function definitions */
extern void part_01();
extern void part_03();
extern void part_05();
extern void part_07();
extern void part_09();
extern void part_11();
extern void part_13();
extern void part_14();
extern void part_15();
extern void part_16();
extern void part_17();
extern void part_19();
extern void part_21();
extern void part_22();
extern void part_23();
extern void part_24();
extern void part_25();
extern void part_27();
extern void part_29();
extern void part_31();
extern void part_33();
extern void part_35();
extern void part_37();
extern void part_41();
extern void part_45();
